/*==================================================================================================
    SCRIPT:     Create Support Views.sql                                                            
    AUTHOR:     Patrick Jasinski, 01/25/2012
==================================================================================================*/

/*==================================================================================================
    DROP OBJECTS
==================================================================================================*/
if OBJECT_ID(N'dbo.DbLayout') is not null drop view dbo.DbLayout
if OBJECT_ID(N'dbo.SimpleDbLayout') is not null drop view dbo.SimpleDbLayout
if OBJECT_ID(N'dbo.Fkeys') is not null drop view dbo.Fkeys
if OBJECT_ID(N'dbo.TableRowCounts') is not null drop view dbo.TableRowCounts
if OBJECT_ID(N'dbo.CodeObjects') is not null drop view dbo.CodeObjects

go
/*==================================================================================================
    OBJECT: DbLayout view
    SOURCE: http://www.jasinskionline.com/TechnicalWiki/Database-Layout-SQL-Server.ashx
==================================================================================================*/
CREATE view [dbo].[DbLayout]
as
select distinct

     DatabaseName   = db_name()
    ,SchemaName     = s.name
    ,TableName      = o.name 
    ,ColumnName     = c.name

    ,DataType       = case 
                        when t.name in ('char','text','varchar')

                            then t.name + '(' + case when c.max_length <= 0 then 'max' else convert(varchar,c.max_length) end + ')'

                        when t.name in ('nchar','ntext','nvarchar')

                            then t.name + '(' + case when c.max_length <= 0 then 'max' else convert(varchar,c.max_length/2) end + ')'

                        when t.name in ('decimal','numeric','real')

                            then t.name + '(' + convert(varchar,c.precision) + ',' + 
                                                convert(varchar,c.scale) + ')'

                        else
 
                            t.name 

                        end

    ,IsNullable     = c.is_nullable 
    ,IsIdentity     = c.is_identity
    
    ,IdentityDesc   = case when c.is_identity = 0 then '' else '(' 
                        + convert(varchar(50), IDENT_SEED(o.name)) 
                        + ', ' 
                        + convert(varchar(50), IDENT_INCR(o.name))
                        + ')' 
                        end

    ,IsPkey         = case when c.column_id in 
                        (select k.colid from 
                            sysindexes i 
                            inner join sysindexkeys k on i.indid = k.indid and i.id = k.id
                            inner join sysobjects o2 on i.id = o2.id 
                            inner join syscolumns c2 on k.id = c2.id and k.colid = c2.colid
                        where (i.status & 0x800) = 0x800
                            and o2.id = o.object_id) then 1 else 0 end

    ,ColumnOrder    = c.column_id

    ,DefaultValue   = coalesce((select definition 
                                from sys.default_constraints 
                                where object_id = c.default_object_id), '')

    ,ColumnDesc     = coalesce((
                        select value 
                        from fn_listextendedproperty('MS_Description', 'schema', s.name, 
                                'table', o.name, 'column', c.name) 
                        ), '')

    ,TableDesc      = coalesce((
                        select value 
                        from fn_listextendedproperty('MS_Description', 'schema', s.name, 
                            'table', o.name, default, default) 
                        ), '')
                        
    ,ObjectType     = o.Type
    ,ColumnType     = t.name
    ,ColumnMaxLen   = case when c.max_length <= 0 then 'max' else convert(varchar,c.max_length) end
    ,IsCalculated   = c.is_computed
    ,Calculation    = cc.definition
   
from 
    sys.objects o

    inner join sys.columns c 
        on o.object_id = c.object_id

    inner join sys.types t 
        on t.system_type_id = c.system_type_id

    inner join sys.schemas s
        on o.schema_id = s.schema_id

    left join sys.computed_columns cc
        on cc.object_id = o.object_id
        and cc.column_id = c.column_id

where 1=1
    and o.type in ('u','v')
    and t.name not in ('sysname')
    and o.name not in ('dtproperties')

GO
/*==================================================================================================
    OBJECT: Fkeys view
    SOURCE: http://www.jasinskionline.com/TechnicalWiki/List-Every-Foreign-Key.ashx
==================================================================================================*/
create view dbo.Fkeys
as
with RefAction as (
    select RefCode = 0, RefDesc = 'Do nothing'
    union select 1, 'Cascade'
    union select 2, 'Set null'
    union select 3, 'Set default'
    )
,fk as (
    select
         ForeignKey         = fk.name
        ,OnSchema           = ots.name 
        ,OnTable            = OnTable.name
        ,AgainstSchema      = ats.name
        ,AgainstTable       = AgainstTable.name
        ,OnColumn           = OnColumn.name
        ,AgainstColumn      = AgainstColumn.name
        ,Enforced           = case when fk.is_disabled = 1 then 'Disabled' else 'Enforced' end
        ,DeleteAction       = da.RefDesc
        ,UpdateAction       = ua.RefDesc

    from 

        sys.foreign_keys fk

        inner join sys.foreign_key_columns fkcols
            on fk.object_id = fkcols.constraint_object_id

        inner join sys.objects OnTable
            on fk.parent_object_id = OnTable.object_id

        inner join sys.schemas ots
            on OnTable.schema_id = ots.schema_id

        inner join sys.objects AgainstTable
            on fk.referenced_object_id = AgainstTable.object_id

        inner join sys.schemas ats
            on AgainstTable.schema_id = ats.schema_id

        inner join sys.columns OnColumn
            on fkcols.parent_column_id = OnColumn.column_id
            and fkcols.parent_object_id = OnColumn.object_id

        inner join sys.columns AgainstColumn
            on fkcols.referenced_column_id = AgainstColumn.column_id
            and fkcols.referenced_object_id = AgainstColumn.object_id

        inner join RefAction da
            on fk.delete_referential_action = da.RefCode

        inner join RefAction ua
            on fk.update_referential_action = ua.RefCode
    )
select
     *
    ,DropCode   = 'alter table [' + OnSchema + '].[' + OnTable + '] drop constraint [' + ForeignKey + ']'
    ,CreateCode = 'alter table ['  + OnSchema + '].[' + OnTable + '] with check add constraint [' + ForeignKey 
                    + '] foreign key ([' + OnColumn + ']) references [' + AgainstSchema + '].[' + AgainstTable
                    + '] ([' + AgainstColumn + '])'
from 
    fk
go
/*==================================================================================================
    OBJECT: TableRowCounts view
    SOURCE: http://www.jasinskionline.com/TechnicalWiki/TableRowCounts.ashx
==================================================================================================*/
create view dbo.TableRowCounts
as
SELECT 
     SchemaName = s.name
    ,TableName  = o.name
    ,RowQty     = sum(p.rows)  
 
FROM
    sys.objects o 

    inner join sys.schemas s
        on o.schema_id = s.schema_id

    inner join sys.partitions p 
        on p.object_id = o.object_id
        and o.type = 'U'

    LEFT JOIN  sys.allocation_units a 
        ON p.partition_id = a.container_id 

WHERE 1=1
    -- 0 = heap table, 1 = table with clustered index
    and p.index_id in (0, 1) 

    and a.type = 1 -- row-data only , not LOB
    and p.rows is not null

group by
    o.name, s.name
go
/*==================================================================================================
    OBJECT: CodeObjects view
==================================================================================================*/
create view dbo.CodeObjects
as

select
     ObjectName         = o.name
    ,ObjectType         = o.type
    ,ObjectTypeDescrip  = o.type_desc
    ,SchemaName         = s.name
    ,SourceCode         = m.definition
    
from    
    sys.objects o
    
    inner join sys.schemas s
        on o.schema_id = s.schema_id
        
    inner join sys.sql_modules m
        on o.object_id = m.object_id
go
